package info.aspirecn.iov.sjjh.supplier10000040.util;

import com.alibaba.fastjson.JSON;
import info.aspirecn.iov.sjjh.supplier10000040.model.DecodeThreeElementRequestObject;
import info.aspirecn.iov.sjjh.supplier10000040.property.SupplierProperties;
import info.aspirecn.rdc.aspirecloud.node.except.utils.ErrorUtils;
import lombok.extern.slf4j.Slf4j;
import okhttp3.*;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.net.SocketTimeoutException;
import java.util.concurrent.TimeUnit;

/**
 * @author xusonglin
 * @version V1.0
 **/
@Slf4j
@Component
public class Md5Sha256DecodeUtil {
    private static SupplierProperties supplierProperties;
    private static OkHttpClient okHttpClient = new OkHttpClient.Builder().build();

    @Autowired
    public Md5Sha256DecodeUtil(SupplierProperties supplierConfig){
        Md5Sha256DecodeUtil.supplierProperties = supplierConfig;
    }
    public static String getDecodeIdCard(String idCode, String name, String decodeType, int outTime) {
        DecodeThreeElementRequestObject.DecodeThreeElementRequestData dataObject = new DecodeThreeElementRequestObject.DecodeThreeElementRequestData();
        if (StringUtils.isNotBlank(idCode)) {
            dataObject.setId_number(idCode.toUpperCase());
        }
        if (StringUtils.isNotBlank(name)) {
            dataObject.setName(name);
        }
        dataObject.setEncrypt(decodeType);
        DecodeThreeElementRequestObject requestObject = new DecodeThreeElementRequestObject();
        requestObject.setApi(supplierProperties.getDecodeApi());
        requestObject.setAppKey(supplierProperties.getDecodeAppKey());
        requestObject.setAppSecret(supplierProperties.getDecodeAppSecret());
        requestObject.setData(dataObject);

        MediaType mediaType = MediaType.parse("application/json; charset=utf-8");
        RequestBody requestBody = RequestBody.create(mediaType, JSON.toJSONString(requestObject));
        Request okHttpRequest = new Request.Builder()
                .post(requestBody)
                .url(supplierProperties.getDecodeUrl())
                .build();
        OkHttpClient decodeClient = okHttpClient.newBuilder()
                .connectTimeout(outTime, TimeUnit.MILLISECONDS)
                .readTimeout(outTime, TimeUnit.MILLISECONDS)
                .writeTimeout(outTime, TimeUnit.MILLISECONDS)
                .build();

        String decodeResponseContext = "";
        decodeResponseContext = getString(okHttpRequest, decodeClient, decodeResponseContext);
        return decodeResponseContext;
    }

    public static String getString(Request okHttpRequest, OkHttpClient decodeClient, String decodeResponseContext) {
        try {
            Response decodeResponse = decodeClient.newCall(okHttpRequest).execute();
            if (decodeResponse.body() != null) {
                decodeResponseContext = decodeResponse.body().string();
            }
            log.info("supplier10000040.scmDecodeIdCard接口decodeResponseContext={}", decodeResponseContext);
        } catch (SocketTimeoutException ste) {
            ErrorUtils.captureException(ste);
            log.info("supplier10000040.getDecodeIdCard接口SocketTimeoutException={}", ste);
        } catch (Exception e) {
            ErrorUtils.captureException(e);
            log.info("supplier10000040.decodeIdCode接口Exception={}", e);
        }
        return decodeResponseContext;
    }
}
